@@ -8,22 +8,28 @@
 #' @title k-Prototypes Clustering
 #' @description Computes k-prototypes clustering for mixed-type data.
 #' 
-#' @details The algorithm like k-means iteratively recomputes cluster prototypes and reassigns clusters.
-#' For \code{type = "standard"}
-#' clusters are assigned using \eqn{d(x,y) =  d_{euclid}(x,y) + \lambda d_{simple\,matching}(x,y)}.
-#' Cluster prototypes are computed as cluster means for numeric variables and modes for factors 
-#' (cf. Huang, 1998).
-#' Ordered factors variables are treated as categorical variables.
-#' In case of \code{na.rm = FALSE}: for each observation variables with missings are ignored 
-#' (i.e. only the remaining variables are considered for distance computation). 
-#' In consequence for observations with missings this might result in a change of variable's weighting compared to the one specified
-#' by \code{lambda}. For these observations distances to the prototypes will typically be smaller as they are based 
-#' on fewer variables.
-#' For \code{type = "gower"} cf. \code{\link{kproto_gower}}.
+#' @details Like k-means, the k-prototypes algorithm iteratively recomputes cluster prototypes and reassigns 
+#' clusters, whereby with \code{type = "huang"} clusters are assigned using the distance
+#' \eqn{d(x,y) =  d_{euclid}(x,y) + \lambda d_{simple\,matching}(x,y)}. Cluster prototypes are computed as 
+#' cluster means for numeric variables and modes for factors (cf. Huang, 1998). Ordered factors variables 
+#' are treated as categorical variables.\cr
+#' For \code{type = "gower"} range-normalized absolute distances from the cluster median are computed for 
+#' the numeric variables (and for the ranks of the ordered factors respectively). For factors simple matching 
+#' distance is used as in the original k prototypes algorithm. The prototypes are given by the median for 
+#' numeric variables, the mode for factors and the level with the closest rank to the median rank of the 
+#' corresponding cluster (cf. Szepannek et al., 2024).\cr
+#' In case of \code{na.rm = FALSE}: for each observation variables with missings are ignored (i.e. only the 
+#' remaining variables are considered for distance computation). In consequence for observations with missings 
+#' this might result in a change of variable's weighting compared to the one specified by \code{lambda}. For 
+#' these observations distances to the prototypes will typically be smaller as they are based on fewer variables.\cr
+#' The \code{type} argument also accepts input \code{"standard"}, but this naming convention is deprecated and 
+#' has been renamed to \code{"huang"}. Please use \code{"huang"} instead.
 #' 
 #' @keywords classif 
 #' @keywords cluster
 #' @keywords multivariate
+#' @importFrom stats median
+#' @importFrom stats complete.cases
 #' 
 #' 
 #' @rdname kproto
@@ -33,21 +39,32 @@ kproto <- function (x, ...)
 
 #' @aliases kproto kproto.default 
 #' 
-#' @param x Data frame with both numerics and factors.
-#' @param k Either the number of clusters, a vector specifying indices of initial prototypes, or a data frame of prototypes of the same columns as \code{x}.
-#' @param lambda Parameter > 0 to trade off between Euclidean distance of numeric variables 
-#' and simple matching coefficient between categorical variables. Also a vector of variable specific factors is possible where 
-#' the order must correspond to the order of the variables in the data. In this case all variables' distances will be multiplied by 
-#' their corresponding lambda value.
-#' @param type Character, to specify the distance for clustering. Either \code{"standard"} (cf. details below) or \code{"gower"}. The latter calls \code{\link{kproto_gower}}.
-#' @param iter.max Maximum number of iterations if no convergence before.
-#' @param nstart If > 1 repetitive computations with random initializations are computed and the result with minimum tot.dist is returned.
-#' @param na.rm Character; Either "yes" to strip NA values for complete case analysis, "no" to keep and ignore NA values, "imp.internal" to impute the NAs within the algorithm or "imp.onestep" to apply the algorithm ignoring the NAs and impute them after the partition is determined.
-#' @param keep.data Logical whether original should be included in the returned object.
-#' @param verbose Logical whether additional information about process should be printed. 
+#' @param x Data frame with both numerics and factors (also ordered factors are possible).
+#' @param k Either the number of clusters, a vector specifying indices of initial prototypes, or a data frame of 
+#' prototypes of the same columns as \code{x}.
+#' @param lambda Parameter > 0 to trade off between Euclidean distance of numeric variables and simple matching 
+#' coefficient between categorical variables (if \code{type = "huang"}). Also a vector of variable specific factors 
+#' is possible where the order must correspond to the order of the variables in the data. In this case all variables' 
+#' distances will be multiplied by their corresponding lambda value.
+#' @param type Character, to specify the distance for clustering. Either \code{"huang"} or \code{"gower"} (cf. details 
+#' below).
+#' @param iter.max Numeric; maximum number of iterations if no convergence before.
+#' @param nstart Numeric; If > 1 repetitive computations with random initializations are computed and the result with 
+#' minimum \code{tot.dist} is returned.
+#' @param na.rm Character, either \code{"yes"} to strip \code{NA} values for complete case analysis, \code{"no"} to 
+#' keep and ignore \code{NA} values, \code{"imp.internal"} to impute the \code{NAs} within the algorithm or 
+#' \code{"imp.onestep"} to apply the algorithm ignoring the \code{NAs} and impute them after the partition is determined.
+#' @param keep.data Logical, whether original should be included in the returned object.
+#' @param verbose Logical, whether additional information about process should be printed. 
 #' Caution: For \code{verbose=FALSE}, if the number of clusters is reduced during the iterations it will not mentioned.
-#' @param init Character, to specify the initialization strategy. Either \code{"nbh.dens"}, \code{"sel.cen"} or \code{"nstart.m"}. Default is \code{"NULL"}, which results in nstart repetitive algorithm computations with random starting prototypes. Otherwise, \code{nstart} is not used. Argument k must be a number if a specific initialization strategy is choosen!
-#' @param p_nstart.m Numeric, probability(=0.9 is default) for \code{init="nstart.m"}, where the strategy assures that with a probability of \code{p_nstart.m} at least one of the m sets of initial prototypes contains objects of every cluster group (cf. Aschenbruck et al. (2023): Random-based Initialization for clustering mixed-type data with the k-Prototypes algorithm. In: {\emph{Cladag 2023 Book of abstracts and short spapers}}, isbn: 9788891935632.).  
+#' @param init Character, to specify the initialization strategy. Either \code{"nbh.dens"}, \code{"sel.cen"} or 
+#' \code{"nstart.m"}. Default is \code{"NULL"}, which results in nstart repetitive algorithm computations with random 
+#' starting prototypes. Otherwise, \code{nstart} is not used. Argument \code{k} must be a number if a specific 
+#' initialization strategy is choosen!
+#' @param p_nstart.m Numeric, probability(=0.9 is default) for \code{init="nstart.m"}, where the strategy assures 
+#' that with a probability of \code{p_nstart.m} at least one of the m sets of initial prototypes contains objects 
+#' of every cluster group (cf. Aschenbruck et al. (2023): Random-based Initialization for clustering mixed-type data 
+#' with the k-Prototypes algorithm. In: {\emph{Cladag 2023 Book of abstracts and short spapers}}, isbn: 9788891935632.).  
 #' @param \dots Currently not used.
 #
 #' @return \code{\link{kmeans}} like object of class \code{kproto}:
@@ -108,13 +125,14 @@ kproto <- function (x, ...)
 #' @author \email{gero.szepannek@@web.de}
 #' 
 #' @references \itemize{
-#'     \item Szepannek, G. (2018): clustMixType: User-Friendly Clustering of Mixed-Type Data in R, {\emph{The R Journal 10/2}}, 200-208, 
-#'           \doi{10.32614/RJ-2018-048}.
+#'     \item Szepannek, G. (2018): clustMixType: User-Friendly Clustering of Mixed-Type Data in R, 
+#'     {\emph{The R Journal 10/2}}, 200-208, \doi{10.32614/RJ-2018-048}.
 #'     \item Aschenbruck, R., Szepannek, G., Wilhelm, A. (2022): Imputation Strategies for Clustering Mixed‑Type Data with Missing Values, 
 #'     {\emph{Journal of Classification}}, \doi{10.1007/s00357-022-09422-y}. 
-#'     \item Z.Huang (1998): 
-#'           Extensions to the k-Means Algorithm for Clustering Large Data Sets with Categorical Variables, 
-#'           Data Mining and Knowledge Discovery 2, 283-304.
+#'     \item Szepannek, G., Aschenbruck, R., Wilhelm, A. (2024): Clustering Large Mixed-Type Data with Ordinal Variables,
+#'     {\emph{Advances in Data Analysis and Classification}}, \doi{10.1007/s11634-024-00595-5}.
+#'     \item Z.Huang (1998): Extensions to the k-Means Algorithm for Clustering Large Data Sets with Categorical Variables, 
+#'           {\emph{Data Mining and Knowledge Discovery 2}}, 283-304.
 #'   }
 #' 
 #' @rdname kproto
@@ -125,7 +143,7 @@ kproto <- function (x, ...)
 #' 
 #' @method kproto default
 #' @export 
-kproto.default <- function(x, k, lambda = NULL, type = "standard", iter.max = 100, nstart = 1, na.rm = "yes", keep.data = TRUE, verbose = TRUE, init = NULL, p_nstart.m = 0.9, ...){
+kproto.default <- function(x, k, lambda = NULL, type = "huang", iter.max = 100, nstart = 1, na.rm = "yes", keep.data = TRUE, verbose = TRUE, init = NULL, p_nstart.m = 0.9, ...){
   
   # enable input of tibbles
   if(is_tibble(x) == TRUE){x <- as.data.frame(x)}
@@ -174,8 +192,11 @@ kproto.default <- function(x, k, lambda = NULL, type = "standard", iter.max = 10
     stop("Argument na.rm must be either 'yes','no','imp.internal' or 'imp.onestep'!")
   }
   
-  if(!type %in% c("standard", "gower")) stop("Argument type must be either 'standard' or 'gower'!")
-  if(type == "standard"){
+  
+  if(type %in% c("Huang", "standard")) type <- "huang"
+  if(type == "Gower") type <- "gower"
+  if(!type %in% c("huang", "gower")) stop("Argument type must be either 'huang' or 'gower'!")
+  if(type == "huang"){
     
     # save origin data, before starting the imputation process
     if(na.rm == "imp.internal"){origin <- x}
@@ -269,17 +290,6 @@ kproto.default <- function(x, k, lambda = NULL, type = "standard", iter.max = 10
       if(verbose) cat("Estimated lambda:", lambda, "\n\n")
     }
     
-    # deleting all incomplete observations
-    if(na.rm == "yes"){
-      miss <- apply(x, 1, function(z) any(is.na(z)))
-      if(verbose){
-        cat(sum(miss), "observation(s) with NAs.\n")
-        if(sum(miss) > 0) message("Observations with NAs are removed.\n")
-        cat("\n")
-      }
-      x <- x[!miss,]
-    }
-    
     # initialize clusters
     clusters  <- numeric(nrow(x)) 
     tot.dists <- NULL
@@ -483,8 +493,8 @@ kproto.default <- function(x, k, lambda = NULL, type = "standard", iter.max = 10
   }
   
   if(type == "gower"){
-    if(na.rm %in% c("imp.internal", "imp.onestep")){
-      stop("Argument na.rm must be either 'yes' or 'no', since imputation is not yet implemented for type = 'gower'!")
+    if(na.rm %in% c("imp.internal")){
+      stop("Argument na.rm must be either 'yes','no' or 'imp-onestep', since imp.internal is not yet implemented for type = 'gower'!")
     }
     res <- kproto_gower(x=x, k=k_input, lambda = lambda, iter.max = iter.max, verbose=verbose, na.rm = na.rm)
   }
@@ -509,12 +519,30 @@ kproto.default <- function(x, k, lambda = NULL, type = "standard", iter.max = 10
   
   if(na.rm == "imp.onestep"){
     x <- res$data
-    if(any(is.na(x[,numvars]))){
-      x[,numvars][is.na(x[,numvars])] <- protos[clusters,numvars][is.na(x[,numvars])]
-    }
-    if(any(is.na(x[,catvars]))){
-      x[,catvars][is.na(x[,catvars])] <- protos[clusters,catvars][is.na(x[,catvars])]
+    
+    if(type == "huang"){
+      if(any(is.na(x[,numvars]))){
+        x[,numvars][is.na(x[,numvars])] <- protos[clusters,numvars][is.na(x[,numvars])]
+      }
+      if(any(is.na(x[,catvars]))){
+        x[,catvars][is.na(x[,catvars])] <- protos[clusters,catvars][is.na(x[,catvars])]
+      }
+    }else{#type=gower
+      numvars <- sapply(x, is.numeric)
+      ordvars <- sapply(x, is.ordered)
+      catvars <- sapply(x, is.factor) & !ordvars
+      
+      if(any(is.na(x[,numvars]))){
+        x[,numvars][is.na(x[,numvars])] <- protos[clusters,numvars][is.na(x[,numvars])]
+      }
+      if(any(is.na(x[,ordvars]))){
+        x[,ordvars][is.na(x[,ordvars])] <- protos[clusters,ordvars][is.na(x[,ordvars])]
+      }
+      if(any(is.na(x[,catvars]))){
+        x[,catvars][is.na(x[,catvars])] <- protos[clusters,catvars][is.na(x[,catvars])]
+      }
     }
+    
     res$data <- x
   }
   
@@ -731,7 +759,7 @@ predict.kproto <-function(object, newdata, ...){
   protos <- object$centers
   x <- newdata
   
-  if(object$type == "standard"){
+  if(object$type == "huang"){
     # check for numeric and factor variables  
     numvars <- sapply(x, is.numeric)
     catvars <- sapply(x, is.factor)
@@ -975,6 +1003,7 @@ clprofiles <- function(object, x, vars = NULL, col = NULL){
 #' 
 #' @importFrom stats var
 #' @importFrom stats sd
+#' @importFrom tibble is_tibble 
 #' @export
 lambdaest <- function(x, num.method = 1, fac.method = 1, outtype = "numeric", verbose = TRUE){
   
@@ -1049,13 +1078,13 @@ print.kproto <- function(x, ...){
   cat("Distance type:", x$type, "\n\n")
   
   cat("Numeric predictors:", sum(sapply(x$centers, is.numeric)), "\n")
-  numord <- sum(sapply(x$centers, is.factor))
+  numord <- sum(sapply(x$centers, is.ordered))
   numcat <- sum(sapply(x$centers, is.factor))
   if(x$type == "gower"){
     cat("Ordinal predictors:", numord, "\n")
     cat("Categorical predictors:", numcat - numord, "\n")
   }
-  if(x$type == "standard") cat("Categorical predictors:", numcat, "\n")
+  if(x$type == "huang") cat("Categorical predictors:", numcat, "\n")
   if(!is.null(x$lambda)) cat("Lambda:", x$lambda, "\n\n")
   
   cat("Number of Clusters:", length(x$size), "\n")
